/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.utils;

// REMPL API classes
import com.rempl.api.Logger;
import com.rempl.api.RuntimeProblem;

// Apache Velocity
import org.apache.velocity.runtime.RuntimeServices;
import org.apache.velocity.runtime.log.LogChute;

/**
 * Broker between Velocity and our {@link Logger}.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: VelocityLogger.java 701 2011-03-16 20:20:40Z yegor256@yahoo.com $
 */
public final class VelocityLogger implements LogChute {

    /**
     * {@inheritDoc}
     */
    @Override
    public void init(final RuntimeServices rsvc) {
        // intentionally empty
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void log(final int level, final String message) {
        this.send(level, message);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void log(final int level, final String message,
        final Throwable thr) {
        this.send(level, message);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean isLevelEnabled(final int level) {
        boolean enabled;
        switch (level) {
            case LogChute.DEBUG_ID:
                enabled = Logger.isDebugEnabled(this);
                break;
            case LogChute.TRACE_ID:
                enabled = Logger.isTraceEnabled(this);
                break;
            default:
                enabled = true;
                break;
        }
        return enabled;
    }

    /**
     * Send message to our logger with the give level.
     * @param level Logging level, defined as an integer constant in
     *              the parent {@link LogChute} class
     * @param message The message to log
     * @see #log(int,String)
     * @see #log(int,String,Throwable)
     */
    private void send(final int level, final String message) {
        switch (level) {
            case LogChute.ERROR_ID:
                Logger.error(this, message);
                break;
            case LogChute.WARN_ID:
                Logger.warn(this, message);
                break;
            case LogChute.INFO_ID:
                Logger.info(this, message);
                break;
            case LogChute.DEBUG_ID:
                Logger.debug(this, message);
                break;
            case LogChute.TRACE_ID:
                Logger.trace(this, message);
                break;
            default:
                throw RuntimeProblem.make("Unknown log level %d", level);
        }
    }

}
