/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.utils;

// REMPL API classes
import com.rempl.api.Logger;
import com.rempl.api.RuntimeProblem;

// string writing
import java.io.StringWriter;

// for velocity props manipulations
import java.util.Map;
import java.util.Properties;

// Apache Velocity
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

/**
 * Writer of texts using Apache Velocity markup system.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id: VelocityWriter.java 706 2011-03-16 20:51:06Z yegor256@yahoo.com $
 * @see <a href="http://velocity.apache.org/engine/releases/velocity-1.7/developer-guide.html">Apache Velocity Developers Guide</a>
 */
public final class VelocityWriter {

    /**
     * The instance of the singleton.
     */
    public static final VelocityWriter INSTANCE =
        new VelocityWriter();

    /**
     * Private ctor.
     */
    private VelocityWriter() {
        this.init();
    }

    /**
     * Initialize Velocity engine.
     * @see VelocityWriter()
     */
    public void init() {
        if (Logger.isTraceEnabled(this)) {
            Logger.trace(this, "#init()");
        }
        final Properties props = new Properties();
        try {
            props.load(
                this.getClass().getResource("/velocity.properties")
                .openStream()
            );
        } catch (java.io.IOException ex) {
            throw RuntimeProblem.make(ex);
        }
        Velocity.init(props);
    }

    /**
     * Create text using template and context.
     * @param name The name of template (resource name)
     * @param ctx Map of context items
     * @return The text written
     */
    public String write(final String name, final Map<String, Object> ctx) {
        if (Logger.isTraceEnabled(this)) {
            Logger.trace(this, "#write(name, ctx)");
        }
        final Template template = Velocity.getTemplate(name);
        final StringWriter writer = new StringWriter();
        template.merge(new VelocityContext(ctx), writer);
        return writer.toString();
    }

}
